@@ -5,7 +5,7 @@
 
     sphinx-build command-line handling.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -22,6 +22,7 @@ from sphinx.errors import SphinxError
 from sphinx.application import Sphinx
 from sphinx.util import Tee, format_exception_cut_frames, save_traceback
 from sphinx.util.console import red, nocolor, color_terminal
+from sphinx.util.pycompat import terminal_safe
 
 
 def usage(argv, msg=None):
@@ -90,6 +91,13 @@ def main(argv):
     if err:
         return 1
 
+    # likely encoding used for command-line arguments
+    try:
+        locale = __import__('locale')  # due to submodule of the same name
+        likely_encoding = locale.getpreferredencoding()
+    except Exception:
+        likely_encoding = None
+
     buildername = None
     force_all = freshenv = warningiserror = use_pdb = False
     status = sys.stdout
@@ -129,7 +137,11 @@ def main(argv):
             try:
                 val = int(val)
             except ValueError:
-                pass
+                if likely_encoding:
+                    try:
+                        val = val.decode(likely_encoding)
+                    except UnicodeError:
+                        pass
             confoverrides[key] = val
         elif opt == '-A':
             try:
@@ -141,7 +153,11 @@ def main(argv):
             try:
                 val = int(val)
             except ValueError:
-                pass
+                if likely_encoding:
+                    try:
+                        val = val.decode(likely_encoding)
+                    except UnicodeError:
+                        pass
             confoverrides['html_context.%s' % key] = val
         elif opt == '-n':
             confoverrides['nitpicky'] = True
@@ -175,8 +191,7 @@ def main(argv):
     except KeyboardInterrupt:
         if use_pdb:
             import pdb
-            print >>error, red('Interrupted while building, '
-                                   'starting debugger:')
+            print >>error, red('Interrupted while building, starting debugger:')
             traceback.print_exc()
             pdb.post_mortem(sys.exc_info()[2])
         return 1
@@ -184,17 +199,17 @@ def main(argv):
         if use_pdb:
             import pdb
             print >>error, red('Exception occurred while building, '
-                                   'starting debugger:')
+                               'starting debugger:')
             traceback.print_exc()
             pdb.post_mortem(sys.exc_info()[2])
         else:
             print >>error
             if isinstance(err, SystemMessage):
                 print >>error, red('reST markup error:')
-                print >>error, err.args[0].encode('ascii', 'backslashreplace')
+                print >>error, terminal_safe(err.args[0])
             elif isinstance(err, SphinxError):
                 print >>error, red('%s:' % err.category)
-                print >>error, err
+                print >>error, terminal_safe(unicode(err))
             else:
                 print >>error, red('Exception occurred:')
                 print >>error, format_exception_cut_frames().rstrip()
