@@ -22,7 +22,8 @@ from sphinx.util.console import darkred, nocolor, color_terminal
 
 
 class BuildDoc(Command):
-    """Distutils command to build Sphinx documentation.
+    """
+    Distutils command to build Sphinx documentation.
 
     The Sphinx build can then be triggered from distutils, and some Sphinx
     options can be set in ``setup.py`` or ``setup.cfg`` instead of Sphinx own
@@ -136,10 +137,7 @@ class BuildDoc(Command):
                      freshenv=self.fresh_env)
 
         try:
-            if self.all_files:
-                app.builder.build_all()
-            else:
-                app.builder.build_update()
+            app.build(force_all=self.all_files)
         except Exception, err:
             from docutils.utils import SystemMessage
             if isinstance(err, SystemMessage):
